# filename: __init__.py.py
# @Time    : 2025/11/3 20:32
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
测试模块：MCP 相关测试 | Test module: MCP related tests

注意：此包仅用于测试，不应与第三方库 mcp 混淆
Note: This package is for testing only and should not be confused with the third-party mcp library
"""
