# filename: __init__.py
# @Time    : 2025/11/04 16:48
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
MCP 资源实现 | MCP Resources Implementation
"""

from ide4ai.a2c_smcp.resources.window import WindowResource

__all__ = ["WindowResource"]
